@@ -3,14 +3,18 @@
  *
  * All rights reserved.
  *
- * This file is part of ObjFW. It may be distributed under the terms of the
- * Q Public License 1.0, which can be found in the file LICENSE.QPL included in
- * the packaging of this file.
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License version 3.0 only,
+ * as published by the Free Software Foundation.
  *
- * Alternatively, it may be distributed under the terms of the GNU General
- * Public License, either version 2 or 3, which can be found in the file
- * LICENSE.GPLv2 or LICENSE.GPLv3 respectively included in the packaging of this
- * file.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
+ * version 3.0 for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * version 3.0 along with this program. If not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -20,7 +24,9 @@
 #import "OFApplication.h"
 #import "OFArray.h"
 #import "OFDate.h"
+#import "OFFile.h"
 #import "OFFileManager.h"
+#import "OFIRI.h"
 #import "OFLocale.h"
 #import "OFNumber.h"
 #import "OFPair.h"
@@ -48,19 +54,21 @@ setPermissions(OFString *path, OFLHAArchiveEntry *entry)
 #ifdef OF_FILE_MANAGER_SUPPORTS_PERMISSIONS
 	OFNumber *POSIXPermissions = entry.POSIXPermissions;
 
-	if (POSIXPermissions == nil)
-		return;
+	if (POSIXPermissions != nil) {
+		OFFileAttributes attributes;
 
-	POSIXPermissions = [OFNumber numberWithUnsignedShort:
-	    POSIXPermissions.unsignedShortValue & 0777];
+		POSIXPermissions = [OFNumber numberWithUnsignedShort:
+		    POSIXPermissions.unsignedShortValue & 0777];
+		attributes = [OFDictionary
+		    dictionaryWithObject: POSIXPermissions
+				  forKey: OFFilePOSIXPermissions];
 
-	OFFileAttributes attributes = [OFDictionary
-	    dictionaryWithObject: POSIXPermissions
-			  forKey: OFFilePOSIXPermissions];
-
-	[[OFFileManager defaultManager] setAttributes: attributes
-					 ofItemAtPath: path];
+		[[OFFileManager defaultManager] setAttributes: attributes
+						 ofItemAtPath: path];
+	}
 #endif
+
+	[app quarantineFile: path];
 }
 
 static void
@@ -85,21 +93,21 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 		app = (OFArc *)[OFApplication sharedApplication].delegate;
 }
 
-+ (instancetype)archiveWithPath: (OFString *)path
-			 stream: (OF_KINDOF(OFStream *))stream
-			   mode: (OFString *)mode
-		       encoding: (OFStringEncoding)encoding
++ (instancetype)archiveWithIRI: (OFIRI *)IRI
+			stream: (OF_KINDOF(OFStream *))stream
+			  mode: (OFString *)mode
+		      encoding: (OFStringEncoding)encoding
 {
-	return [[[self alloc] initWithPath: path
-				    stream: stream
-				      mode: mode
-				  encoding: encoding] autorelease];
+	return [[[self alloc] initWithIRI: IRI
+				   stream: stream
+				     mode: mode
+				 encoding: encoding] autorelease];
 }
 
-- (instancetype)initWithPath: (OFString *)path
-		      stream: (OF_KINDOF(OFStream *))stream
-			mode: (OFString *)mode
-		    encoding: (OFStringEncoding)encoding
+- (instancetype)initWithIRI: (OFIRI *)IRI
+		     stream: (OF_KINDOF(OFStream *))stream
+		       mode: (OFString *)mode
+		   encoding: (OFStringEncoding)encoding
 {
 	self = [super init];
 
@@ -240,7 +248,7 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 					[OFStdOut writeLine: OF_LOCALIZED(
 					    @"list_osid",
 					    @"Operating system identifier: "
-					    "%[osid]",
+					    @"%[osid]",
 					    @"osid", OSID)];
 				}
 			}
@@ -445,16 +453,10 @@ outer_loop_end:
 }
 
 - (void)addFiles: (OFArray OF_GENERIC(OFString *) *)files
+  archiveComment: (OFString *)archiveComment
 {
 	OFFileManager *fileManager = [OFFileManager defaultManager];
 
-	if (files.count < 1) {
-		[OFStdErr writeLine: OF_LOCALIZED(@"add_no_file_specified",
-		    @"Need one or more files to add!")];
-		app->_exitStatus = 1;
-		return;
-	}
-
 	for (OFString *fileName in files) {
 		void *pool = objc_autoreleasePoolPush();
 		OFFileAttributes attributes;
